#!/bin/bash

export BASE_DIR=../../../../
export LIB_DIR=$BASE_DIR/Libraries/
export SRC_DIR=$BASE_DIR/WrapperSources/

export CUSTOM_DEFINES=
export CUSTOM_FLAGS=
export CUSTOM_LDFLAGS=

lowercase() {
  echo $1 | tr "[:upper:]" "[:lower:]" 
}

vercomp () {
    if [[ $1 == $2 ]]; then
        return 1
    fi
    local IFS=.
    local i ver1=($1) ver2=($2)
    # fill empty fields in ver1 with zeros
    for ((i=${#ver1[@]}; i<${#ver2[@]}; i++)); do
        ver1[i]=0
    done
    for ((i=0; i<${#ver1[@]}; i++)); do
        if [[ -z ${ver2[i]} ]]; then
            ver2[i]=0
        fi
        if ((10#${ver1[i]} > 10#${ver2[i]})); then
            return 2
        fi
        if ((10#${ver1[i]} < 10#${ver2[i]})); then
            return 0
        fi
    done
    return 1
}

if [ "$1" == "g" -o "$1" == "g++" ]; then

  echo Compile sample using g++ compiler

  export CPP=g++

  # check if C++0x/C++11 features are supported
  export VER=`g++ -dumpversion`
  vercomp $VER "4.7"
  if [ $? -ge 1 ]; then
    export CUSTOM_FLAGS+=" -std=c++11"
    export CUSTOM_LDFLAGS+=" -std=c++11"
  else
    vercomp $VER "4.4"
    if [ $? -ge 1 ]; then
      export CUSTOM_FLAGS+=" -std=c++0x"
      export CUSTOM_LDFLAGS+=" -std=c++0x"
    else
      export CUSTOM_DEFINES+=" -DSB_DEFINE_CHAR16"
    fi
  fi

  OS=`lowercase \`uname\``
  if [ "$OS" == "darwin" ]; then
    export LIB_DIR+="MacOS"
    export CUSTOM_DEFINES+=" -DMACOS"
  else
    export LIB_DIR+="Linux"
    export CUSTOM_DEFINES+=" -DLINUX"
    export CUSTOM_LDFLAGS+=" -static-libgcc -Wl,--gc-sections -Wl,--print-gc-sections -Wl,--strip-all"
  fi

  if [ `uname -m` == 'x86_64' -a "$3" != "x86" ]; then
    export CUSTOM_DEFINES+=" -DCPU64"
    export LIB_DIR+="64"
    export CUSTOM_FLAGS+=" -m64"
    export CUSTOM_LDFLAGS+=" -m64"
  else
    export LIB_DIR+="32"
    export CUSTOM_FLAGS+=" -m32"
    export CUSTOM_LDFLAGS+=" -m32"
  fi

  export LIB_DIR+="/"

  if [ "$2" == "static" ]; then
    #Using a precompiled static library
    export STATIC_LIB=sbbex
    make -f Makefile_static.gpp
  else
    make -f Makefile.gpp
  fi

  exit

fi

if [ "$1" == "clang" -o "$1" == "clang++" ]; then

  echo Compile sample using clang++ compiler

  export CPP=clang++

  # check if C++0x/C++11 features are supported
  export VER=`clang++ --version 2>&1 | grep version | awk '{print $4}'`
  vercomp $VER "3.1"
  if [ $? -ge 1 ]; then
    export CUSTOM_FLAGS+=" -std=c++11 -stdlib=libc++"
    export CUSTOM_LDFLAGS+=" -std=c++11 -stdlib=libc++"
  else
    vercomp $VER "3.0"
    if [ $? -ge 1 ]; then
      export CUSTOM_FLAGS+=" -std=c++0x -stdlib=libc++"
      export CUSTOM_LDFLAGS+=" -std=c++0x -stdlib=libc++"
    else
      export CUSTOM_DEFINES+=" -DSB_DEFINE_CHAR16"
    fi
  fi

  OS=`lowercase \`uname\``
  if [ "$OS" == "darwin" ]; then
    export LIB_DIR+="MacOS"
    export CUSTOM_DEFINES+=" -DMACOS"
  else
    export LIB_DIR+="Linux"
    export CUSTOM_DEFINES+=" -DLINUX"
    export CUSTOM_LDFLAGS+=" -Wl,--gc-sections -Wl,--print-gc-sections -Wl,--strip-all"
  fi

  if [ `uname -m` == 'x86_64' -a "$3" != "x86" ]; then
    export CUSTOM_DEFINES+=" -DCPU64"
    export LIB_DIR+="64"
    export CUSTOM_FLAGS+=" -m64"
    export CUSTOM_LDFLAGS+=" -m64"
  else
    export LIB_DIR+="32"
    export CUSTOM_FLAGS+=" -m32"
    export CUSTOM_LDFLAGS+=" -m32"
  fi

  export LIB_DIR+="/"

  if [ "$2" == "static" ]; then
    #Using a precompiled static library
    export STATIC_LIB=sbbex
    make -f Makefile_static.gpp
  else
    make -f Makefile.gpp
  fi

  exit

fi

echo Usage: makefile.sh compiler [static]
echo Compilers:
echo  g, g++ - GNU C++ compiler
echo  clang, clang++ - LLVM compiler 
echo Options:
echo  static - use a precompiled static library

read input

if [[ $input != "" ]]; then
  echo Running: ./makefile.sh $input
  ./makefile.sh $input
fi

exit
