#include <cstring>
#include <iostream>
#include <fstream>
#include <stdio.h>
#include <stdlib.h>
#include <algorithm>
#include <ctime>

#include "sbmisc.h"
#include "sbxmlcore.h"
#include "sbxmlsig.h"
#include "sbxmlsec.h"
#include "sbhashfunction.h"
#include "sbpublickeycrypto.h"
#include "sbpgpkeys.h"
#include "sbx509.h"

using namespace SecureBlackbox;

std::string UnQuote(std::string s)
{
    if (s.length() < 1)
        return s;

	s.erase(remove( s.begin(), s.end(), '\"' ), s.end());

    return s;
}

void SB_CALLBACK XML_OnFormatElement(void * /*_ObjectData*/, TObjectHandle /*hSender*/, TElXMLDOMElementHandle hElement, int32_t Level, const sb_char16_t * pcPath, int32_t szPath, sb_char16_t * pcStartTagWhitespace, int32_t * szStartTagWhitespace, sb_char16_t * pcEndTagWhitespace, int32_t * szEndTagWhitespace)
{
	try
	{
#ifdef SB_U16STRING_USED
		sb_u16string u16StartTagWhitespace(pcStartTagWhitespace, *szStartTagWhitespace);
		std::wstring StartTagWhitespace;
		sb_u16_to_wstring(u16StartTagWhitespace, StartTagWhitespace);
#else
		std::wstring StartTagWhitespace(pcStartTagWhitespace, *szStartTagWhitespace);
#endif
		StartTagWhitespace = L"\n";
		std::wstring s(Level - 1, '\t');

		StartTagWhitespace += s;
		SBSetEventReturnStringW(5, StartTagWhitespace);

		TElXMLDOMElement element(hElement, false);
		if (element.get_FirstChild() != NULL)
		{
			bool HasElements = false;
			TElXMLDOMNode* Node = element.get_FirstChild();
			while (Node != NULL)
			{
				if (Node->get_NodeType() == SB_ntElement)
				{
					HasElements = true;
					break;
				}

				Node = Node->get_NextSibling();
			}

			if (HasElements)
			{
#ifdef SB_U16STRING_USED
				sb_u16string u16EndTagWhitespace(pcEndTagWhitespace, *szEndTagWhitespace);
				std::wstring EndTagWhitespace;
				sb_u16_to_wstring(u16EndTagWhitespace, EndTagWhitespace);
#else
				std::wstring EndTagWhitespace(pcEndTagWhitespace, *szEndTagWhitespace);
#endif
				EndTagWhitespace = L"\n" + s;
				SBSetEventReturnStringW(6, EndTagWhitespace);
			}
		}
	}
	catch (SBException E)
	{
		std::cout << "Unexpected error in OnFormatElement event handler" << std::endl;
		std::cout << E.what() << std::endl;
		std::cout << "Stack trace: " << E.getErrorStackTrace().c_str() << std::endl;
	}
	catch (...)
	{
		std::cout << "Unexpected error in OnFormatElement event handler" << std::endl;
	}
}

void SB_CALLBACK XML_OnFormatText(void * /*_ObjectData*/, TObjectHandle /*Sender*/, sb_char16_t * pcText, int32_t * szText, TElXMLTextTypeRaw TextType, int32_t Level, const sb_char16_t * pcPath, int32_t szPath)
{
	try
	{
#ifdef SB_U16STRING_USED
		sb_u16string u16text(pcText, *szText);
		std::wstring text;
		sb_u16_to_wstring(u16text, text);
#else
		std::wstring text(pcText, *szText);
#endif
		if ((TextType == ttBase64) && (text.length() > 64))
		{
			std::wstring s = L"\n";
			while (text.length() > 0)
			{
				if (text.length() > 64)
				{
					s += text.substr(0, 64);
					s += L"\n";
					text.erase(0, 64);
				}
				else
				{
					s = s + text + L"\n";
					text = L"";
				}
			}

			std::wstring ss(Level - 2, '\t');
			text = s + ss;

			SBSetEventReturnStringW(2, text);
		}
	}
	catch (SBException E)
	{
		std::cout << "Unexpected error in OnFormatText event handler" << std::endl;
		std::cout << E.what() << std::endl;
		std::cout << "Stack trace: " << E.getErrorStackTrace().c_str() << std::endl;
	}
	catch (...)
	{
		std::cout << "Unexpected error in OnFormatText event handler" << std::endl;
	}
}

void LoadCertificate(TFileStream &F, std::string &Password, TElXMLKeyInfoX509Data &X509KeyData)
{
	F.set_Position(0);
	TElX509Certificate cert(NULL);
	int k = cert.LoadFromStreamAuto(F, Password, 0);
	if (k == 0)
	{
		X509KeyData.set_Certificate(cert);
		cert.updateHandle(NULL); // detach wrapped class handle from wrapper class, so it won't be freed
	}
}

bool DoSign(TElXMLDOMDocument &FXMLDocument, std::string &xmlFilename, TElXMLSignatureType signType, TElXMLCanonicalizationMethod canonMethod, TElXMLSigMethodType signMethodType,
	TElXMLSignatureMethod signMethod, TElXMLMACMethod MACMethod, std::string &keyFilename, std::string &passphrase)
{
	TElXMLSigner Signer(NULL);
	TElXMLKeyInfoHMACData HMACKeyData(true);
	TElXMLKeyInfoRSAData RSAKeyData(true);
	TElXMLKeyInfoX509Data X509KeyData(true);
	TElXMLKeyInfoPGPData PGPKeyData(true);
	try
	{
		/* version 11 */
		TElXMLReference Ref;
		Ref.set_DigestMethod(xdmSHA1);
		if (signType == xstEnveloped)
		{
			// create a reference for a document element with enveloped signature transform
			Ref.set_URI(L"");
			Ref.set_URINode(FXMLDocument.get_DocumentElement());
			TElXMLEnvelopedSignatureTransform Transform;
			Ref.get_TransformChain()->Add(Transform);
			Transform.updateHandle(NULL);
		}
		else if (signType == xstEnveloping)
		{
			Signer.set_EnvelopingObjectID(L"idObj");
			Ref.set_URI(L"#idObj");			
		}
		else // detached
		{
			Ref.set_URI(std::wstring(xmlFilename.begin(), xmlFilename.end()) + L"#");
			Ref.set_URINode(FXMLDocument.get_DocumentElement());
		}

		Signer.get_References()->Add(Ref);
		Ref.updateHandle(NULL); // detach wrapped class handle from wrapper class, so it won't be freed
		/**/
		/* version 12
		int k = Signer.get_References()->Add();
		TElXMLReference *Ref = Signer.get_References()->get_Reference(k);
		Ref->set_DigestMethod(xdmSHA1);
		if (signType == xstEnveloped)
		{
			Ref->set_URI(L"");
			Ref->set_URINode(FXMLDocument.get_DocumentElement());
			if (signType == xstEnveloped)
				Ref->get_TransformChain()->AddEnvelopedSignatureTransform();
		}
		else if ...
		*/

		Signer.set_SignatureType(signType);
		Signer.set_CanonicalizationMethod(canonMethod);
		Signer.set_SignatureMethodType(signMethodType);
		Signer.set_SignatureMethod(signMethod);
		Signer.set_MACMethod(MACMethod);
		Signer.set_KeyName(L"");
		Signer.set_IncludeKey(true);

		Signer.set_OnFormatElement(&XML_OnFormatElement, NULL);
		Signer.set_OnFormatText(&XML_OnFormatText, NULL);

		if (keyFilename == "")
		{
			std::cout << "Failed to sign: No Key" << std::endl;
			return false;
		}

		if (signMethodType == xmtMAC)
		{
			TFileStream stream(keyFilename, filemodeOpenRead);

			std::vector<uint8_t> Buf((size_t)stream.get_Size());
			if (stream.get_Size() > 0)
				stream.Read(&Buf.front(), (int32_t)stream.get_Size());

			TElHMACKeyMaterial* key = HMACKeyData.get_Key();
			key->set_Key_ByteArray(Buf);
			Signer.set_KeyData(HMACKeyData);
		}
		else
		{
			TElRSAKeyMaterial* key = RSAKeyData.get_RSAKeyMaterial();
			key->set_Passphrase(passphrase);

			TFileStream stream(keyFilename, filemodeOpenRead);

			std::vector<uint8_t> Buf((size_t)stream.get_Size());
			if (stream.get_Size() > 0)
				stream.Read(&Buf.front(), (int32_t)stream.get_Size());

			try
			{
				key->LoadPublic(&Buf.front(), (int32_t)stream.get_Size());
			}
			catch (SBException E) {}

			if (!key->get_PublicKey())
			{
				stream.set_Position(0);
				try
				{
					key->LoadSecret(&Buf.front(), (int32_t)stream.get_Size());
				}
				catch (SBException E) {}
			}

			if (!key->get_PublicKey())
			{
				stream.set_Position(0);
				LoadCertificate(stream, passphrase, X509KeyData);
			}

			if (!key->get_PublicKey() && (X509KeyData.get_Certificate() == NULL))
			{
				TElPGPPublicKey pubKey;
				try
				{
					stream.set_Position(0);
					pubKey.LoadFromStream(stream);
					// if loaded ok
					PGPKeyData.set_PublicKey(pubKey);
					pubKey.updateHandle(NULL); // detach wrapped class handle from wrapper class, so it won't be freed
				}
				catch (SBException E) {}

				if (PGPKeyData.get_PublicKey() == NULL)
				{
					TElPGPSecretKey secKey;
					secKey.set_Passphrase(passphrase);
					try
					{
						stream.set_Position(0);
						secKey.LoadFromStream(stream);
						// if loaded ok
						PGPKeyData.set_SecretKey(secKey);
						secKey.updateHandle(NULL); // detach wrapped class handle from wrapper class, so it won't be freed
					}
					catch (SBException E) {}
				}
			}

			if (RSAKeyData.get_RSAKeyMaterial()->get_PublicKey())
				Signer.set_KeyData(RSAKeyData);
			else if (X509KeyData.get_Certificate() != NULL)
				Signer.set_KeyData(X509KeyData);
			else if ((PGPKeyData.get_PublicKey() != NULL) || (PGPKeyData.get_SecretKey() != NULL))
				Signer.set_KeyData(PGPKeyData);
			else
			{
				std::cout << "Failed to load Key" << std::endl;
				return false;
			}
		}

		if (signType != xstEnveloping) 
			Signer.UpdateReferencesDigest();

		Signer.GenerateSignature();

		if (signType == xstDetached)
		{
			// detached signature is always placed in a new xml document
			TElXMLDOMNode SigNode(NULL, false);

			try
			{				
				Signer.Save(SigNode);

				// release Document object
				FXMLDocument.Free();
				// replace with newly created Document
				FXMLDocument.updateHandle(SigNode.get_OwnerDocument()->getHandle());
			}
			catch (SBException E)
			{
				std::cout << "Failed to sign data and to save the signature: " << E.what() << std::endl;
				return false;
			}
		}
		else
		{
			// initialize SigNode with a document element (where a signature will be placed)
			TElXMLDOMNode SigNode(FXMLDocument.get_DocumentElement()->getHandle(), false);

			try
			{
				// If the signature type is enveloping, then the signature is placed into the passed node and the contents of the node are moved to inside of the signature.
				// If the signature type is enveloped, the signature is placed as a child of the passed node.
				Signer.Save(SigNode);
			}
			catch (SBException E)
			{
				std::cout << "Failed to sign data and to save the signature: " << E.what() << std::endl;
				return false;
			}
		}
	}
	catch (SBException E)
	{
		std::cout << "Error: " << E.what() << std::endl;
		return false;
	}

	return true;
}


int main(int argc, char **argv)
{
	std::string helpMessage = "consolesigner.exe [params] [flags]\n" \
		"[params]\n" \
		"-in : input xml filename\n" \
		"-out : output xml filename\n" \
		"-k : key filename\n" \
		"-p : passphrase\n" \
		"[flags]\n" \
		"-st : signature type [enveloped | enveloping | detached] default - enveloped\n" \
		"-smt : signature method type [mac | sig] default - sig\n" \
		"-cm : canonicalization method [canonical | canonical_comments | minimal] default - canonical\n" \
		"-sm : signature method [DSS | RSA_SHA1 | RSA_MD5 | RSA_SHA256 | RSA_SHA384 | RSA_SHA512 | RSA_RIPEMD160 ] default - RSA_SHA1\n" \
		"-hmac : mac method [HMAC_MD5 | HMAC_SHA1 | HMAC_SHA224 | HMAC_SHA256 | HMAC_SHA384 | HMAC_SHA512 | HMAC_RIPEMD160] default - HMAC_SHA256\n";

	std::string xmlFilenameIn, xmlFilenameOut, keyFilename, keyPassphrase;
	TElXMLSignatureType signType = xstEnveloped;
	TElXMLCanonicalizationMethod canonMethod = xcmCanon;
	TElXMLSignatureMethod signMethod = xsmRSA_SHA1;
	TElXMLMACMethod MACMethod = xmmHMAC_SHA256;
	TElXMLSigMethodType signMethodType = xmtSig;

	try
	{
		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
SetLicenseKey("<put the key here>");
	}
	catch (SBException E)
	{
		std::cout << "Failed to set license key:" << std::endl;
		std::cout << E.what() << std::endl;
		return 0;
	}

	if (argc < 2)
	{
		std::cout << helpMessage << std::endl;
		return 0;
	}

	try
	{
		int i = 1;
		while (i < argc)
		{
			std::string param;
			param.assign(argv[i]);

			if (param == "-in")
			{
				if (++i <= argc)
					xmlFilenameIn = UnQuote(argv[i]);
			}
			else if (param == "-out")
			{
				if (++i <= argc)
					xmlFilenameOut = UnQuote(argv[i]);
			}
			else if (param == "-k")
			{
				if (++i <= argc)
					keyFilename = UnQuote(argv[i]);
			}
			else if (param == "-p")
			{
				if (++i <= argc)
					keyPassphrase = UnQuote(argv[i]);
			}
			else if (param == "-st")
			{
				if (++i <= argc)
				{
					std::string value = UnQuote(argv[i]);
					if (value == "enveloped")
					{
						signType = xstEnveloped;
					}
					else if (value == "enveloping")
					{
						signType = xstEnveloping;
					}
					else if (value == "detached")
					{
						signType = xstDetached;
					}
				}
			}
			else if (param == "-smt")
			{
				if (++i <= argc)
				{
					std::string value = UnQuote(argv[i]);
					if (value == "mac")
					{
						signMethodType = xmtMAC;
					}
					else if (value == "sig")
					{
						signMethodType = xmtSig;
					}
				}
			}
			else if (param == "-cm")
			{
				if (++i <= argc)
				{
					std::string value = UnQuote(argv[i]);
					if (value == "canonical")
					{
						canonMethod = xcmCanon;
					}
					else if (value == "canonical_comments")
					{
						canonMethod = xcmCanonComment;
					}
					else if (value == "minimal")
					{
						canonMethod = xcmMinCanon;
					}
				}
			}
			else if (param == "-sm")
			{
				if (++i <= argc)
				{
					std::string value = UnQuote(argv[i]);
					if (value == "DSS")
					{
						signMethod = xsmDSS;
					}
					else if (value == "RSA_SHA1")
					{
						signMethod = xsmRSA_SHA1;
					}
					else if (value == "RSA_MD5")
					{
						signMethod = xsmRSA_MD5;
					}
					else if (value == "RSA_SHA256")
					{
						signMethod = xsmRSA_SHA256;
					}
					else if (value == "RSA_SHA384")
					{
						signMethod = xsmRSA_SHA384;
					}
					else if (value == "RSA_SHA512")
					{
						signMethod = xsmRSA_SHA512;
					}
					else if (value == "RSA_RIPEMD160")
					{
						signMethod = xsmRSA_RIPEMD160;
					}
				}
			}
			else if (param == "-hmac")
			{
				if (++i <= argc)
				{
					std::string value = UnQuote(argv[i]);
					if (value == "HMAC_MD5")
					{
						MACMethod = xmmHMAC_MD5;
					}
					else if (value == "HMAC_SHA1")
					{
						MACMethod = xmmHMAC_SHA1;
					}
					else if (value == "HMAC_SHA224")
					{
						MACMethod = xmmHMAC_SHA224;
					}
					else if (value == "HMAC_SHA256")
					{
						MACMethod = xmmHMAC_SHA256;
					}
					else if (value == "HMAC_SHA384")
					{
						MACMethod = xmmHMAC_SHA384;
					}
					else if (value == "HMAC_SHA512")
					{
						MACMethod = xmmHMAC_SHA512;
					}
					else if (value == "HMAC_RIPEMD160")
					{
						MACMethod = xmmHMAC_RIPEMD160;
					}
				}
			}
			else if (param == "-h")
			{
				std::cout << helpMessage << std::endl;
			}

			i++;
		}
	}
	catch (...)
	{
		std::cout << "Failed to parse parameters" << std::endl;
		std::cout << helpMessage << std::endl;
		return 0;
	}

	TElXMLDOMDocument FXMLDocument;
	try
	{
		std::cout << "Loading XML document: " << xmlFilenameIn << std::endl;
		FXMLDocument.LoadFromFile(xmlFilenameIn);
	}
	catch (SBException E)
	{
		std::cout << "Failed to load XML document: " << E.what() << std::endl;
		return 0;
	}

	try
	{
		if (!DoSign(FXMLDocument, xmlFilenameIn, signType, canonMethod, signMethodType, signMethod, MACMethod, keyFilename, keyPassphrase))
			return 0;
	}
	catch (SBException E)
	{
		std::cout << "Failed to sign xml document: " << E.what() << std::endl;
	}

	try
	{
		std::cout << "Saving XML document: " << xmlFilenameOut << std::endl;
		FXMLDocument.SaveToFile(xmlFilenameOut);
	}
	catch (SBException E)
	{
		std::cout << "Failed to save XML document: " << E.what() << std::endl;
		return 0;
	}
}