#include <cstring>
#include <iostream>
#include <fstream>
#include <stdio.h>
#include <stdlib.h>
#include <algorithm>

#include "sbutils.h"
#include "sbarczip.h"
#include "sbziputils.h"

using namespace SecureBlackbox;

int FLastProgress;
bool passwordAsked = false;

std::string ReadLine()
{
	std::string result;
	std::getline(std::cin, result);
	return result;
}

std::string ReadPassword()
{
	return ReadLine();
}

void SB_CALLBACK Zip_CompressionStartEvent(void * /* objData */, TObjectHandle /* hSender */, TElZipArchiveDirectoryEntryHandle Entry, int8_t * Compress)
{
	std::string compression, path;
	TElZipArchiveDirectoryEntry dir (Entry, false);
	ZipCompressionMethodName(dir.get_CompressionMethod(), compression);
	dir.get_Path(path);
    std::cout << "Compressing " << path << " (" << compression << ") [";
    FLastProgress = 0;
}

void SB_CALLBACK Zip_CompressionFinishedEvent(void * /* objData */, TObjectHandle /* hSender */, TElZipArchiveDirectoryEntryHandle Entry)
{
    for (int i = FLastProgress + 1; i <= 10; i++)
        std::cout << '.';

    std::cout << "] OK" << std::endl;
}

void SB_CALLBACK Zip_ProgressEvent(void * /* objData */, TObjectHandle /* hSender */, uint64_t Processed, uint64_t Total, uint64_t OverallProcessed, uint64_t OverallTotal, int8_t * Cancel)
{
    int i, NewProgress;

    NewProgress = (int)((Processed * 10) / Total);
    if (NewProgress > FLastProgress)
    {
        for (i = FLastProgress + 1; i <= NewProgress; i++)
            std::cout << '.';
        FLastProgress = NewProgress;
    }
}

void SB_CALLBACK Zip_ExtractionStartEvent(void *  objData, TObjectHandle /* hSender */, TElZipArchiveDirectoryEntryHandle Entry, int8_t * Extract)
{
	TElZipArchiveDirectoryEntry dir (Entry, false);
    if (dir.get_Encrypted() && (!passwordAsked))
    {
		TElZipWriter *zipWriter = (TElZipWriter *)objData;
        std::cout << "Password is needed: " << std::endl;
        zipWriter->set_Password(ReadPassword());
        passwordAsked = true;
    }

	std::string compression, path;
	ZipCompressionMethodName(dir.get_CompressionMethod(), compression);
	dir.get_Path(path);
    std::cout << "Extracting " << path << " (" << compression << ") [" << std::endl;
    FLastProgress = 0;
    *Extract = true;
}

void SB_CALLBACK Zip_ExtractionFinished(void * /* objData */, TObjectHandle /* hSender */, TElZipArchiveDirectoryEntryHandle Entry)
{
    for (int i = FLastProgress + 1; i <= 10; i++)
        std::cout << '.';

    std::cout << "] OK" << std::endl;            
}

void SB_CALLBACK Zip_PasswordNeededEvent(void * /* objData */, TObjectHandle /* hSender */, void * Param, char * pcPassword, int32_t * szPassword, int8_t * Cancel)
{
    std::cout << "Password is needed: " << std::endl;
	std::string password = ReadPassword();
	SBSetEventFirstReturnStringA(password);
    passwordAsked = true;
}

void SB_CALLBACK Zip_UserActionNeededEvent(void * /* objData */, TObjectHandle /* hSender */, int32_t ForEvent, const char * pcDescription, int32_t szDescription, void * Param, int32_t * UserAction)
{
	std::string desc (pcDescription, szDescription);
    // user actions handling 
    if (ForEvent == SB_ZIP_EVENT_CRC_MISMATCH)
    {
        std::cout << desc << std::endl;
        *UserAction = SB_ZIP_ACTION_IGNORE;
    }
    else if (ForEvent == SB_ZIP_EVENT_FILE_ALREADY_EXISTS)
        *UserAction = SB_ZIP_ACTION_IGNORE;
    else if (ForEvent == SB_ZIP_EVENT_DIR_ALREADY_EXISTS)
        *UserAction = SB_ZIP_ACTION_IGNORE;
    else
        *UserAction = SB_ZIP_ACTION_ABORT;
}

std::string GetEncryptionInfo(TElZipWriter &zipWriter)
{
    std::string result;
    if (!zipWriter.get_Encrypt())
        result = "none";
    else if (!(zipWriter.get_StrongEncryption() || zipWriter.get_WinZipEncryption()))
        result = "weak";
	else if (zipWriter.get_WinZipEncryption())
	{
		int keySize = zipWriter.get_WinZipAesKeySize();
		result = "WinZip, AES-" + std::to_string((long long)keySize);
	}
    else if (zipWriter.get_StrongEncryption())
    {  
		TElZipStrongEncryptionInfo* sei = zipWriter.get_StrongEncryptionInfo();
		std::string enc;
		ZipEncryptionAlgorithmName(sei->get_EncryptionAlgorithm(), enc);
        result = "Strong encryption, " + enc;
        if (zipWriter.get_DirectoryEncrypted())
            result = result + ", encrypted directory";
    }

    return result;
}

std::string UnQuote(std::string s)
{
    if (s.length() < 1)
        return s;

	s.erase(remove( s.begin(), s.end(), '\"' ), s.end());

    return s;
}

int main(int argc, char **argv)
{
	bool compressMode, decompressMode = false;
	std::string helpMessage = "Usage: consolezip.exe [flags] [operation] input1, input2, .... [-o output]\n\n" \
		"Where possible operations are: \n" \
		"-c : compress input files (or directories, wildcards allowed), and save archive to output\n" \
		"-d : decompress input archives (no wildcards!), and write files to output directory\n" \
		"Where possible flags are:\n" \
		"-e [encryption mode]: encrypt with [encryption mode], where possible modes are: weak, winzip [aes128|aes192|aes256], strong [aes128|aes192|aes256|3des] [dir]\n" \
		"-m [deflate|bzip2|stored] [1..9]: set compression algorithm and compression level\n" \
		"-p password: set password for encryption/decryption";
	std::string compressionLevels = "123456789";
	std::string outputPath;
	int i;

	try
	{
		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
SetLicenseKey("<put the key here>");
	}
	catch (SBException E)
	{
		std::cout << "Failed to set license key:" << std::endl;
		std::cout << E.what() << std::endl;
	}

	std::vector<std::string> inputFiles(NULL);
	compressMode = false;
	decompressMode = false;
	passwordAsked = false;
	outputPath = "";

	TElZipWriter zipWriter(NULL);
	zipWriter.set_OnCompressionStart(Zip_CompressionStartEvent, NULL);
	zipWriter.set_OnCompressionFinished(Zip_CompressionFinishedEvent, NULL);
	zipWriter.set_OnProgress(Zip_ProgressEvent, NULL);
	zipWriter.set_OnExtractionStart(Zip_ExtractionStartEvent, &zipWriter);
	zipWriter.set_OnExtractionFinished(Zip_ExtractionFinished, NULL);
	zipWriter.set_OnPasswordNeeded(Zip_PasswordNeededEvent, NULL);
	zipWriter.set_OnUserActionNeeded(Zip_UserActionNeededEvent, NULL);

	std::cout << "EldoS ZipBlackbox console demo." << std::endl;

	try
	{
		try
		{
			std::string param;

			i = 1;
			while (i < argc)
			{
				char *arg = argv[i];
				std::string param;
				param.assign(argv[i]);

				// compression or decompression - input files list
				if ((!strcmp(arg, "-c") || !strcmp(arg, "-d")) && (i < argc - 1))
				{
					if (compressMode || decompressMode)
						throw new SBException();

					if (!strcmp(arg, "-c"))
						compressMode = true;
					else
						decompressMode = true;

					while ((i + 1 < argc) && (argv[i + 1][0] != '-'))
					{
						i++;
						std::string name;
						name.assign(argv[i]);
						inputFiles.push_back(name);
					}
				}
				// encryption options
				else if (!strcmp(arg, "-e"))
				{
					zipWriter.set_Encrypt(true);

					while ((i + 1 < argc) && (argv[i + 1][0] != '-'))
					{
						TElZipStrongEncryptionInfo* sei;

						i++;
						param.assign(argv[i]);

						if (param == "weak")
						{
							zipWriter.set_StrongEncryption(false);
							zipWriter.set_WinZipEncryption(false);
						}
						else if (param == "strong")
						{
							zipWriter.set_StrongEncryption(true);
							zipWriter.set_WinZipEncryption(false);
						}
						else if (param == "winzip")
						{
							zipWriter.set_StrongEncryption(false);
							zipWriter.set_WinZipEncryption(true);
						}
						else if (param == "aes128")
						{
							zipWriter.set_WinZipAesKeySize(128);
							sei = zipWriter.get_StrongEncryptionInfo();
							sei->set_EncryptionAlgorithm(SB_ZIP_ENCRYPTION_AES128);
						}
						else if (param == "aes192")
						{
							zipWriter.set_WinZipAesKeySize(192);
							sei = zipWriter.get_StrongEncryptionInfo();
							sei->set_EncryptionAlgorithm(SB_ZIP_ENCRYPTION_AES192);
						}
						else if (param == "aes256")
						{
							zipWriter.set_WinZipAesKeySize(256);
							sei = zipWriter.get_StrongEncryptionInfo();
							sei->set_EncryptionAlgorithm(SB_ZIP_ENCRYPTION_AES256);
						}
						else if (param == "3des")
						{
							sei = zipWriter.get_StrongEncryptionInfo();
							sei->set_EncryptionAlgorithm(SB_ZIP_ENCRYPTION_3DES168);
						}
						else if (param == "dir")
						{
							zipWriter.set_DirectoryCompressed(true);
							zipWriter.set_DirectoryEncrypted(true);
						}
						else
							throw new SBException();
					}
				}
				else if (param == "-m")
				{
					while ((i + 1 < argc) && (argv[i + 1][0] != '-'))
					{
						i++;
						arg = argv[i];
						param = argv[i];

						if (param == "deflate")
							zipWriter.set_CompressionAlgorithm(SB_ZIP_COMPRESSION_DEFLATE);
						else if (param == "bzip2")
							zipWriter.set_CompressionAlgorithm(SB_ZIP_COMPRESSION_BZIP2);
						else if (param == "stored")
							zipWriter.set_CompressionAlgorithm(SB_ZIP_COMPRESSION_STORED);
						else if (atoi(arg) > 0)
							zipWriter.set_CompressionLevel(atoi(arg));
						else
							throw new SBException();
					}
				}
				else if (param == "-p")
				{
					if (i >= argc)
						throw new SBException();

					zipWriter.set_Password(UnQuote(argv[++i]));
				}
				else if (param == "-o")
				{
					if (i >= argc)
						throw new SBException();

					outputPath = UnQuote(argv[++i]);
				}
				else if (param == "-h")
				{
					std::cout << helpMessage << std::endl;
					std::string res;
					std::getline(std::cin, res);
				}
				else
					throw new SBException();

				i++;
			}
		}
		catch (SBException E)
		{
			std::cout << "Parameters parsing error:" << E.what() << std::endl;
			std::cout << helpMessage << std::endl;
			return 0;
		}

		// creating archive
		if (compressMode)
		{
			std::string password;
			zipWriter.get_Password(password);
			if (zipWriter.get_Encrypt() && (password.length() == 0))
			{
				std::cout << "Enter password to encrypt archive:" << std::endl;
				zipWriter.set_Password(ReadPassword());
				zipWriter.get_Password(password);
				if (password.length() == 0)
				{
					std::cout << "Empty password: encryption is disabled" << std::endl;
					zipWriter.set_Encrypt(false);
				}
			}

			if (outputPath.length() == 0)
				outputPath = "archive.zip";

			std::string compression;

			std::cout << "Creating archive " << outputPath << std::endl;
			std::cout << "Compression algorithm: " << compression << std::endl;
			std::cout << "Encryption: " << GetEncryptionInfo(zipWriter) << std::endl;

			try
			{
				zipWriter.CreateArchive();

				for (int i = 0; i < inputFiles.size(); i++)
				{
					zipWriter.Add(inputFiles[i]);
				}
				zipWriter.Compress(outputPath);
			}
			catch (SBException E)
			{
				std::cout << "Compression error : " << E.what() << std::endl;
				return 0;
			}
		}
		else if (decompressMode)
		{
			if (outputPath.length() == 0)
				outputPath = ".";

			try
			{
				for (i = 0; i < inputFiles.size(); i++)
				{
					zipWriter.Open(inputFiles[i], true);
					zipWriter.Extract("*.*", outputPath);
					zipWriter.Close();
				}
			}
			catch (SBException E)
			{
				std::cout << "Decompression error : " << E.what() << std::endl;
				return 0;
			}
		}
		else
			std::cout << "No operation specified. Use consolezip.exe -h to see the list of available options.";
	}
	catch (SBException E)
	{
		std::cout << "Error : " << E.what() << std::endl;
	}

	//ReadLine();

	return 0;
}